#!/usr/bin/env python

"""
Handlers for a person for whom scheduling is performed, inspecting outgoing
messages to obtain scheduling done externally.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.content import Handler, uri_item
from vCalendar import to_node

class PersonHandler(Handler):

    "Handling mechanisms specific to people."

    def _get_identity(self, objtype, from_organiser=True):

        """
        Get the identity of interest in a usable form for any unprocessed
        object.
        """

        identity, attr = item = uri_item(self.get_item(from_organiser and "ORGANIZER" or "ATTENDEE"))

        # Check for event using UID.

        if not self.have_new_object(identity, objtype):
            return None

        return item

    def _record(self, objtype, from_organiser=True, update_freebusy=False):

        "Record free/busy and object information."

        item = self._get_identity(objtype, from_organiser)
        if not item:
            return False

        identity, attr = item

        # Store the object.

        self.store.set_event(identity, self.uid, to_node(
            {objtype : [(self.details, {})]}
            ))

        # Update free/busy information.

        if update_freebusy:

            # If newer than any old version, discard old details from the
            # free/busy record and check for suitability.

            periods = self.get_periods()
            freebusy = self.store.get_freebusy(identity) or []

            if from_organiser or attr.get("PARTSTAT") != "DECLINED":
                self.update_freebusy(freebusy, identity, periods)
            else:
                self.remove_from_freebusy(freebusy, identity)

            if self.publisher:
                self.publisher.set_freebusy(identity, freebusy)

        return True

    def _remove(self, objtype, from_organiser=True):

        "Remove free/busy information for any unprocessed object."

        item = self._get_identity(objtype, from_organiser)
        if not item:
            return False

        identity, attr = item

        freebusy = self.store.get_freebusy(identity) or []
        self.remove_from_freebusy(freebusy, identity)

        return True

class Event(PersonHandler):

    "An event handler."

    def add(self):
        pass

    def cancel(self):
        self._remove("VEVENT", True)

    def counter(self):
        pass

    def declinecounter(self):
        pass

    def publish(self):
        self._record("VEVENT", True, True)

    def refresh(self):
        self._record("VEVENT", True, True)

    def reply(self):
        self._record("VEVENT", False, True)

    def request(self):
        self._record("VEVENT", True, True)

class Freebusy(PersonHandler):

    "A free/busy handler."

    def publish(self):
        pass

    def reply(self):
        pass

    def request(self):
        pass

class Journal(PersonHandler):

    "A journal entry handler."

    def add(self):
        pass

    def cancel(self):
        self._remove("VJOURNAL", True)

    def publish(self):
        self._record("VJOURNAL", True)

class Todo(PersonHandler):

    "A to-do item handler."

    def add(self):
        pass

    def cancel(self):
        self._remove("VTODO", True)

    def counter(self):
        pass

    def declinecounter(self):
        pass

    def publish(self):
        self._record("VTODO", True)

    def refresh(self):
        self._record("VTODO", True)

    def reply(self):
        self._record("VTODO", False)

    def request(self):
        self._record("VTODO", True)

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
